#include "AmadeusTraceAVR.h"
#include <util/delay.h>
#include <string.h>

#define ATClkDDR   DDRB
#define ATClkPIN   PINB
#define ATClkPORT  PORTB
#define ATClkBit   7
#define ATDataDDR  DDRB
#define ATDataPIN  PINB
#define ATDataPORT PORTB
#define ATDataBit  6

#if F_CPU < 9555555
#define ATWait
#elif F_CPU < 10444444
#define ATWait asm volatile  ("nop");
#elif F_CPU < 11333333
#define ATWait asm volatile  ("nop");asm volatile  ("nop");
#elif F_CPU < 12333333
#define ATWait _delay_loop_1(1);
#elif F_CPU < 13222222
#define ATWait _delay_loop_1(1);asm volatile  ("nop");
#elif F_CPU < 14111111
#define ATWait _delay_loop_1(1);asm volatile  ("nop");asm volatile  ("nop");
#elif F_CPU < 15000001
#define ATWait _delay_loop_1(2);
#elif F_CPU < 16000001
#define ATWait _delay_loop_1(2);asm volatile  ("nop");
#elif F_CPU < 17666666
#define ATWait _delay_loop_1(3);
#elif F_CPU < 18555555
#define ATWait _delay_loop_1(3);asm volatile  ("nop");
#elif F_CPU < 20333333
#define ATWait _delay_loop_1(4);
#else
#define ATWait _delay_loop_1(10);
#endif


void _ATMakeTristate(){
	ATClkDDR   &= ~(1<<ATClkBit);		// Make Clk an Input
	ATDataDDR  &= ~(1<<ATDataBit);		// Make Data an Input

	ATClkPORT  |= (1<<ATClkBit);		// Activate weak pull-up resistor for Clk
	ATDataPORT |= (1<<ATDataBit);		// Activate weak pull-up resistor for Data
}

void _ATActivate(){
	ATClkDDR   |= (1<<ATClkBit);		// Make Clk an Output
	ATDataDDR  |= (1<<ATDataBit);		// Make Data an Output
}



void ATInit(){
	_ATMakeTristate();
	// delay 2x 10ms because max. delay of _delay_ms @ 20MHz is 13ms, for lower F_CPU one _delay_ms might be sufficient
	_delay_ms(10);
	_delay_ms(10);
}

#define _ATTransferBit1(_byte, _bit)\
	ATDataPORT&=~(1<<ATDataBit);\
	if (_byte&(1<<_bit))\
		ATDataPORT|=(1<<ATDataBit);\
	ATClkPORT &= ~(1<<ATClkBit);

#if F_CPU>8666666
#define _ATTransferBit2 	asm volatile ("nop");
#else
#define _ATTransferBit2
#endif

#define _ATTransferBit3 	ATClkPORT |= (1<<ATClkBit);


int ATTraceAllowed(){
	return ATClkPIN&(1<<ATClkBit);
}

void ATSendByte(unsigned char b){
	_ATTransferBit1 (b, 7);
	_ATTransferBit2;
	_ATTransferBit3;
	ATWait;
	_ATTransferBit1 (b, 6);
	_ATTransferBit2;
	_ATTransferBit3;
	ATWait;
	_ATTransferBit1 (b, 5);
	_ATTransferBit2;
	_ATTransferBit3;
	ATWait;
	_ATTransferBit1 (b, 4);
	_ATTransferBit2;
	_ATTransferBit3;
	ATWait;
	_ATTransferBit1 (b, 3);
	_ATTransferBit2;
	_ATTransferBit3;
	ATWait;
	_ATTransferBit1 (b, 2);
	_ATTransferBit2;
	_ATTransferBit3;
	ATWait;
	_ATTransferBit1 (b, 1);
	_ATTransferBit2;
	_ATTransferBit3;
	ATWait;
	_ATTransferBit1 (b, 0);
	_ATTransferBit2;
	_ATTransferBit3;
	ATWait;
}

	
void ATSendString(char* s)
{
	int l;
	if (ATTraceAllowed()){
		_ATActivate();
			
		l = strlen(s);
		ATSendByte(l+1);
		ATSendByte(255);

		while (*s){
			ATSendByte(*s++);
		}
		_ATMakeTristate();
	}	
}
